#!/bin/bash
#
# Gentoo-based package build script

# find out where the build script is located
tdir=`echo "$0" | sed 's%[\\/][^\\/][^\\/]*$%%'`
test "x$tdir" = "x$0" && tdir=.
scriptdir=`cd $tdir; pwd`

# find src directory.
# If scriptdir ends in SPECS, then topdir is $scriptdir/..
# If scriptdir ends in CYGWIN-PATCHES, then topdir is $scriptdir/../..
# Otherwise, we assume that topdir = scriptdir
topdir1=`echo ${scriptdir} | sed 's%/SPECS$%%'`
topdir2=`echo ${scriptdir} | sed 's%/CYGWIN-PATCHES$%%'`
if [ "x$topdir1" != "x$scriptdir" ] ; then # SPECS
	T=`cd ${scriptdir}/..; pwd`
elif [ "x$topdir2" != "x$scriptdir" ] ; then # CYGWIN-PATCHES
		T=`cd ${scriptdir}/../..; pwd`
else
		T=`cd ${scriptdir}; pwd`
fi

tscriptname=`basename $0 .sh`
PN=`echo $tscriptname | sed -e 's/\-[^\-]*\-[^\-]*$//'`
PN1=${PN}
PN2=${PN}-devel
PN3=${PN}-doc
PN4=libIDL
PV=`echo $tscriptname | sed -e "s/${PN}\-//" -e 's/\-[^\-]*$//'`
PR=`echo $tscriptname | sed -e "s/${PN}\-${PV}\-//"`
P=${PN}-${PV}
P1=${PN1}-${PV}
P2=${PN2}-${PV}
P3=${PN3}-${PV}
P4=${PN4}-0.6.8
PVR=${P}-${PR}
PVR1=${P1}-${PR}
PVR2=${P2}-${PR}
PVR3=${P3}-${PR}
PVR4=${P4}-${PR}

# Package maintainers: if the original source is not distributed as a
# (possibly compressed) tarball, set the value of ${src_orig_pkg_name},
# and redefine the unpack() helper function appropriately.
#
# determine correct decompression option and tarball filename
src_orig_pkg_name=
if [ -e "${src_orig_pkg_name}" ] ; then
	opt_decomp=? # Make sure tar punts if unpack() is not redefined
elif [ -e ${P}.tar.bz2 ] ; then
	opt_decomp=j
	src_orig_pkg_name=${P}.tar.bz2
elif [ -e ${P}.tar.gz ] ; then
	opt_decomp=z
	src_orig_pkg_name=${P}.tar.gz
elif [ -e ${P}.tgz ] ; then
	opt_decomp=z
	src_orig_pkg_name=${P}.tgz
elif [ -e ${P}.tar ] ; then
	opt_decomp=
	src_orig_pkg_name=${P}.tar
else
	echo Cannot find original package.
	exit 1
fi

src_orig_pkg=${T}/${src_orig_pkg_name}

# determine correct names for generated files
src_pkg_name=${PVR1}-src.tar.bz2
src_patch_name=${PVR1}.patch
bin_pkg1_name=${PVR1}.tar.bz2
bin_pkg2_name=${PVR2}.tar.bz2
bin_pkg3_name=${PVR3}.tar.bz2
bin_pkg4_name=${PVR4}.tar.bz2
src_pkg=${T}/${src_pkg_name}
src_patch=${T}/${src_patch_name}
bin_pkg1=${T}/${bin_pkg1_name}
bin_pkg2=${T}/${bin_pkg2_name}
bin_pkg3=${T}/${bin_pkg3_name}
bin_pkg4=${T}/${bin_pkg4_name}

S=${T}/${P}
B=${S}/.build
D=${S}/.inst
C=${S}/CYGWIN-PATCHES
srcinstdir=${S}/.sinst
checkfile=${T}/${PVR1}.check

MY_CFLAGS="-O2 -pipe"
MY_LDFLAGS=""
MAKEOPTS="-j2"
SIG=1
MULTIPKG=1

PATCHED_FILES="\
	\
"

install_docs="\
	ABOUT-NLS \
	ANNOUNCE \
	ANNOUNCEMENTS \
	AUTHOR \
	AUTHORS \
	BUG-REPORTS \
	BUGS \
	Changes \
	CONTRIBUTORS \
	COPYING \
	COPYING-DOCS \
	COPYING.LIB \
	COPYRIGHT \
	CREDITS \
	ChangeLog \
	FAQ \
	GPL \
	HACKING \
	HOW-TO-CONTRIBUTE \
	INSTALL \
	KNOWNBUG \
	LEGAL \
	LICENCE \
	LICENSE \
	MAINTAINERS \
	NEWS \
	NLS \
	NOTES \
	PROGLIST \
	README \
	THANKS \
	TODO \
	WHATSNEW \
"

# helper function
# unpacks the original package source archive into ./${P}/
# change this if the original package was not tarred
# or if it doesn't unpack to a correct directory
unpack() {
	tar xv${opt_decomp}f "$@"
}

dodir() {
	for d in "$@" ; do
		mkdir -p ${D}/${d} ;
	done
}

doicon() {
	dodir /usr/share/pixmaps
	for i in "$@" ; do
		/bin/install -m 644 ${i} ${D}/usr/share/pixmaps ;
	done
}

make_desktop() {
	dodir /usr/share/applications

	echo "[Desktop Entry]
Encoding=UTF-8
Version=0.9.2
Name=${2}
Type=Application
Comment=${5}
Exec=${1}
Path=
Icon=${3}
Categories=Application;${4};" > ${D}/usr/share/applications/${1}.desktop
}

mkdirs() {
	cd ${T}
	rm -fr ${B} ${D} ${srcinstdir}
	mkdir -p ${B} ${D} ${srcinstdir} ${C}
}

getsrc() {
	PVP=(${PV//[-\._]/ })
	wget http://ftp.gnome.org/pub/gnome/sources/${PN}/${PVP[0]}.${PVP[1]}/${P}.tar.bz2 || exit 1
}

prepsrc() {
	cd ${T}
	unpack ${src_orig_pkg}
	if [ -f ${src_patch} ] ; then
		patch -Z -p0 < ${src_patch}
	fi
	mkdirs
}

# Run this to re-autotool AFTER editing configure.{ac,in}/Makefile.am
autotool() {
	cd ${S}
	autoconf-2.13
	autoheader-2.13
	automake-1.4 -a -c
	cd ${T}
	if [ -f ${P}/INSTALL ] ; then
		unpack ${src_orig_pkg} ${P}/INSTALL
	fi
}

cygconf() {
	cd ${B}
	CFLAGS="${MY_CFLAGS}" CXXFLAGS="${MY_CFLAGS}" LDFLAGS="${MY_LDFLAGS}" \
	${S}/configure \
		--srcdir="${S}" \
		--sysconfdir=/etc \
		--prefix=/usr \
		--exec-prefix=/usr \
		--bindir=/usr/bin \
		--includedir=/usr/include \
		--libdir=/usr/lib \
		--libexecdir=/usr/libexec \
		--sbindir=/usr/sbin \
		--datadir=/usr/share \
		--infodir=/usr/share/info \
		--mandir=/usr/share/man \
		--localstatedir=/var/lib \
		--disable-gtk-doc \
		--disable-schemas-install \
		"$@" || exit 1
	/bin/install -m 755 ${C}/libtool ${B}/libtool
	/bin/install -m 755 ${C}/libtool ${B}/libIDL/libtool
	/bin/install -m 755 ${C}/libtool ${B}/popt/libtool
}

reconf() {
	cd ${T}
	rm -fr ${B}
	mkdir -p ${B}
	cygconf
}

cygmake() {
	cd ${B}
	make ${MAKEOPTS} "$@"
}

check() {
	cd ${B}
	if [ -f ${S}/Makefile.am ] ; then
		make check "$@" | tee ${checkfile} 2>&1
	else
		make test "$@" | tee ${checkfile} 2>&1
	fi
}

clean() {
	cd ${B} && make clean
}

cyginstall() {
	cd ${B}
	rm -fr ${D}/*
	make install DESTDIR=${D} "$@"
}

cygdoc() {
	cd ${S}
	mkdir -p ${D}/usr/share/doc/Cygwin
	mkdir -p ${D}/usr/share/doc/${P1}
	mkdir -p ${D}/usr/share/doc/${P4}

	for f in ${install_docs} ; do
		if [ -f ${S}/${f} ] ; then
			/bin/install -m 644 ${S}/${f} ${D}/usr/share/doc/${P1}
		fi
	done

	for f in ${install_docs} ; do
		if [ -f ${S}/${PN4}/${f} ] ; then
			/bin/install -m 644 ${S}/${PN4}/${f} ${D}/usr/share/doc/${P4}
		fi
	done

	if [ -f ${C}/README ] ; then
		/bin/install -m 644 ${C}/README \
			${D}/usr/share/doc/Cygwin/${P1}.README
	fi

	if [ -f ${C}/${PN1}.README ] ; then
		/bin/install -m 644 ${C}/${PN1}.README \
			${D}/usr/share/doc/Cygwin/${P1}.README
	fi

	if [ -f ${C}/${PN2}.README ] ; then
		/bin/install -m 644 ${C}/${PN2}.README \
			${D}/usr/share/doc/Cygwin/${P2}.README
	fi

	if [ -f ${C}/${PN3}.README ] ; then
		/bin/install -m 644 ${C}/${PN3}.README \
			${D}/usr/share/doc/Cygwin/${P3}.README
	fi

	if [ -f ${C}/${PN4}.README ] ; then
		/bin/install -m 644 ${C}/${PN4}.README \
			${D}/usr/share/doc/Cygwin/${P4}.README
	fi
}

cygetc() {
	cd ${S}
	for d in postinstall preremove profile.d ; do
		if [ -f ${C}/${d}.sh ] ; then
			mkdir -p ${D}/etc/${d}
			/bin/install -m 755 ${C}/${d}.sh ${D}/etc/${d}/${PN1}.sh
		fi
	done

	if [ -f ${C}/profile.d.csh ] ; then
		mkdir -p ${D}/etc/profile.d
		/bin/install -m 755 ${C}/profile.d.csh \
			${D}/etc/profile.d/${PN1}.csh
	fi
}

prepallinfo() {
	cd ${D}
	for f in /usr/share/info/dir /usr/info/dir ; do
		if [ -f ${D}${f} ] ; then
			rm -f ${D}${f}
		fi
	done
	if [ -d ${D}/usr/share/info ] ; then
		dodir /etc/postinstall
		for i in `find usr/share/info -type f ! -name "*.gz"` ; do
			echo "gzipping info page: `basename $i`" ;
			gzip -q $i ;
			echo "install-info --dir-file=/usr/share/info/dir --info-file=/$i.gz" \
				>> ${D}/etc/postinstall/${PN4}.sh
		done
	fi
}

prepallman() {
	cd ${D}
	if [ -d ${D}/usr/share/man ] ; then
		for m in `find usr/share/man -type f ! -name "*.gz"` ; do
			echo "gzipping man page: `basename $m`" ;
			gzip -q $m ;
		done ;
	fi
}

prepstrip() {
	cd ${D}
	for e in `find * -name "*.dll" -or -name "*.exe"` ; do
		echo "Stripping executable: $e" ;
		strip $e 2>&1 ;
	done
}

list_files() {
	cd ${D}
	find . -name "*" ! -type d | sed 's%^\.%  %'
	true
}

list_deps() {
	cd ${D}
	find ${D} -name "*.exe" -o -name "*.dll" | xargs -r cygcheck | \
		sed -e '/\.exe/d' -e 's,\\,/,g' | sort -bu | xargs -n1 cygpath -u \
		| xargs -r cygcheck -f | sort -bu | sed 's%^%  %'
	true
}

cygpkg() {
	cd ${D}
	tar cvjf ${bin_pkg1} * \
		--exclude="etc" \
		--exclude="usr/bin/*-config" \
		--exclude="usr/bin/*IDL*" \
		--exclude="usr/bin/orbit-idl.exe" \
		--exclude="usr/include" \
		--exclude="usr/lib" \
		--exclude="usr/share/aclocal" \
		--exclude="usr/share/doc/Cygwin/libIDL*" \
		--exclude="usr/share/doc/libIDL*" \
		--exclude="usr/share/idl" \
		--exclude="usr/share/info"
	echo
	tar cvjf ${bin_pkg2} \
		usr/bin/*-config \
		usr/bin/orbit-idl.exe \
		usr/include/ \
		usr/lib/ \
		usr/share/aclocal \
		usr/share/idl \
		--exclude="usr/bin/*IDL*" \
		--exclude="usr/include/*IDL*" \
		--exclude="usr/lib/*IDL*" \
		--exclude="usr/lib/pkgconfig/*IDL*" \
		--exclude="usr/share/aclocal/*IDL*"
	echo
	tar cvjf ${bin_pkg4} \
		etc \
		usr/bin/*IDL* \
		usr/include/libIDL* \
		usr/lib/libIDL* \
		usr/lib/pkgconfig/libIDL* \
		usr/share/aclocal/libIDL* \
		usr/share/doc/Cygwin/libIDL* \
		usr/share/doc/libIDL* \
		usr/share/info/libIDL*
}

mkpatch() {
	cd ${S}
	find . -name "autom4te.cache" | xargs rm -rf
	unpack ${src_orig_pkg}
	mv ${P} ../${P}-orig
	cd ${T}
	rm -f ${srcinstdir}/${src_patch_name}
	diff -urN -x '.build' -x '.inst' -x '.sinst' \
		${P}-orig ${P} > ${srcinstdir}/${src_patch_name}
	rm -rf ${P}-orig
}

cygspkg() {
	mkpatch
	if [ "${SIG}" -eq 1 ] ; then
		name=${srcinstdir}/${src_patch_name} text="PATCH" sigfile
	fi
	cp ${src_orig_pkg} ${srcinstdir}/${src_orig_pkg_name}
	if [ -e ${src_orig_pkg}.sig ] ; then
		cp ${src_orig_pkg}.sig ${srcinstdir}/
	fi
	cp $0 ${srcinstdir}/`basename $0`
	name=$0 text="SCRIPT" sigfile
	if [ "${SIG}" -eq 1 ] ; then
		cp $0.sig ${srcinstdir}/
	fi
	cd ${srcinstdir}
	tar cvjf ${src_pkg} *
}

dist() {
    mkdir -p ${T}/${PN1}
    cp ${src_pkg} ${T}/${PN1}
    mv ${bin_pkg1} ${T}/${PN1}
    cp ${C}/setup.hint ${T}/${PN1}/setup.hint

    if [ -f ${bin_pkg2} ] ; then
        mkdir -p ${T}/${PN1}/${PN2}
        mv ${bin_pkg2} ${T}/${PN1}/${PN2}
        cp ${C}/devel.hint ${T}/${PN1}/${PN2}/setup.hint
    fi

    if [ -f ${bin_pkg3} ] ; then
        mkdir -p ${T}/${PN1}/${PN3}
        mv ${bin_pkg3} ${T}/${PN1}/${PN3}
        cp ${C}/doc.hint ${T}/${PN1}/${PN3}/setup.hint
    fi

    if [ -f ${bin_pkg4} ] ; then
        mkdir -p ${T}/${PN1}/${PN4}
        mv ${bin_pkg4} ${T}/${PN1}/${PN4}
        cp ${C}/idl.hint ${T}/${PN1}/${PN4}/setup.hint
    fi
}

finish() {
	rm -rf ${S}
}

cleandir() {
	rm -fr ${src_orig_pkg} ${src_orig_pkg}.sig $0.sig ${src_patch} \
	${src_patch}.sig
	finish
}

sigfile() {
	if [ \( "${SIG}" -eq 1 \) -a \( -e $name \) -a \( \( ! -e $name.sig \) -o \( $name -nt $name.sig \) \) ] ; then
		if [ -x /usr/bin/gpg ] ; then
			echo "$text signature needs to be updated"
			rm -f $name.sig
			/usr/bin/gpg --detach-sign $name
		else \
			echo "You need the gnupg package installed in order to make signatures."
		fi
	fi
}

checksig() {
	if [ -x /usr/bin/gpg ] ; then
		if [ -e ${src_orig_pkg}.sig ] ; then
			echo "ORIGINAL PACKAGE signature follows:"
			/usr/bin/gpg --verify ${src_orig_pkg}.sig ${src_orig_pkg}
		else \
			echo "ORIGINAL PACKAGE signature missing."
		fi
		if [ -e $0.sig ] ; then
			echo "SCRIPT signature follows:"
			/usr/bin/gpg --verify $0.sig $0
		else \
			echo "SCRIPT signature missing."
		fi
		if [ -e ${src_patch}.sig ] ; then
			echo "PATCH signature follows:"
			/usr/bin/gpg --verify ${src_patch}.sig ${src_patch}
		else \
			echo "PATCH signature missing."
		fi
	else
		echo "You need the gnupg package installed in order to check signatures."
	fi
}

while test -n "$1" ; do
	case $1 in
		download|wget|get)
			getsrc ; STATUS=$? ;;
		prep|unpack)
			prepsrc ; STATUS=$? ;;
		mkdirs)
			mkdirs ; STATUS=$? ;;
		auto*)
			autotool ; STATUS=$? ;;
		conf*)
			cygconf ; STATUS=$? ;;
		reconf)
			reconf ; STATUS=$? ;;
		build|make)
			cygmake ; STATUS=$? ;;
		compile)
			 cygconf && cygmake ; STATUS=$? ;;
		check|test)
			check ; STATUS=$? ;;
		clean)
			clean ; STATUS=$? ;;
		inst*)
			cyginstall && cygdoc && cygetc
			prepallinfo && prepallman && prepstrip ; STATUS=$? ;;
		list)
			list_files ; STATUS=$? ;;
		dep*)
			list_deps ; STATUS=$? ;;
		strip)
			prepstrip ; STATUS=$? ;;
		package|pkg)
			cygpkg ; STATUS=$? ;;
		mkpatch)
			mkpatch ; STATUS=$? ;;
		src-package|spkg)
			cygspkg ; STATUS=$? ;;
		dist)
			dist ; STATUS=$? ;;
		finish)
			finish ; STATUS=$? ;;
		checksig)
			checksig ; STATUS=$? ;;
		first|ready)
			mkdirs && cygspkg && finish ; STATUS=$? ;;
		all)
			checksig && prepsrc &&  cygconf && cygmake
			cyginstall && cygdoc && cygetc && prepallinfo && prepallman
			prepstrip && cygpkg && cygspkg && finish ; STATUS=$? ;;
		*)
			echo "Error: bad arguments" ; exit 1 ;;
	esac
	( exit ${STATUS} ) || exit ${STATUS}
	shift
done
